@@ -1,4 +1,6 @@
 context('parsing functions')
+library(tidyverse)
+library(stringr)
 example <- readVcf(.testfile("vcf4.2.example.sv.vcf"), "")
 simple <- readVcf(.testfile("simple.vcf"), "")
 breakend <- readVcf(.testfile("breakend.vcf"), "")
@@ -212,12 +214,12 @@ test_that("pairs_round_trip", {
     pairs = import(system.file("extdata", f, package = "StructuralVariantAnnotation"))
     gr = pairs2breakpointgr(pairs)
     pairs2 = breakpointgr2pairs(gr)
-    expect_equal(seqnames(first(pairs)), seqnames(first(pairs2)))
-    expect_equal(start(first(pairs)), start(first(pairs2)))
-    expect_equal(strand(first(pairs)), strand(first(pairs2)))
-    expect_equal(seqnames(second(pairs)), seqnames(second(pairs2)))
-    expect_equal(start(second(pairs)), start(second(pairs2)))
-    expect_equal(strand(second(pairs)), strand(second(pairs2)))
+    expect_equal(seqnames(S4Vectors::first(pairs)), seqnames(S4Vectors::first(pairs2)))
+    expect_equal(start(S4Vectors::first(pairs)), start(S4Vectors::first(pairs2)))
+    expect_equal(strand(S4Vectors::first(pairs)), strand(S4Vectors::first(pairs2)))
+    expect_equal(seqnames(S4Vectors::second(pairs)), seqnames(S4Vectors::second(pairs2)))
+    expect_equal(start(S4Vectors::second(pairs)), start(S4Vectors::second(pairs2)))
+    expect_equal(strand(S4Vectors::second(pairs)), strand(S4Vectors::second(pairs2)))
     expect_equal(mcols(pairs)$name, mcols(pairs2)$name)
     expect_equal(mcols(pairs)$score, mcols(pairs2)$score)
   }
@@ -299,10 +301,162 @@ test_that("findInsDupOverlaps", {
 	expect_equal(subjectHits(hits12), queryHits(hits21))
 	expect_equal(subjectHits(hits21), queryHits(hits12))
 })
-
-
-
-
-
-
-
+if (FALSE) {
+	transitiveGr = bpgr
+	subjectGr = bpgr
+	maximumInsertSize=700
+	maximumTransitiveBreakpoints=4
+	positionalMargin=8
+	impreciseTransitiveCalls=(transitiveGr$HOMLEN == 0 | is.null(transitiveGr$HOMLEN)) & start(transitiveGr) != end(transitiveGr)
+	impreciseSubjectCalls=(subjectGr$HOMLEN == 0 | is.null(subjectGr$HOMLEN)) & start(subjectGr) != end(subjectGr)
+	allowImprecise=FALSE
+}
+test_that("findTransitiveCalls imprecise", {
+	bpgr = breakpointRanges(.testrecord(c(
+		"chr1	100	bp1_1	N	N[chr2:200[	.	.	SVTYPE=BND;MATEID=bp1_2",
+		"chr2	200	bp1_2	N	]chr1:100]N	.	.	SVTYPE=BND;MATEID=bp1_1",
+		"chr2	300	bp2_1	N	N[chr3:500[	.	.	SVTYPE=BND;MATEID=bp2_2",
+		"chr3	500	bp2_2	N	]chr2:300]N	.	.	SVTYPE=BND;MATEID=bp2_1",
+		"chr1	50	imprecise_transitive_1	N	N[chr3:530[	.	.	SVTYPE=BND;MATEID=imprecise_transitive_2;IMPRECISE;CIPOS=-49,100",
+		"chr3	530	imprecise_transitive_2	N	]chr1:50]N	.	.	SVTYPE=BND;MATEID=imprecise_transitive_1;IMPRECISE;CIPOS=-100,100"
+	)))
+	transdf = findTransitiveCalls(bpgr, bpgr)
+	resultdf = DataFrame(
+			transitive_breakpoint_name=c("imprecise_transitive_1", "imprecise_transitive_2"),
+			total_distance=101,
+			traversed_breakpoint_names=CharacterList(c("bp1_1", "bp2_1"), c("bp2_2", "bp1_2")),
+			distance_to_traversed_breakpoint=IntegerList(c(0, 101), c(0, 101)))
+	expect_equal(transdf, resultdf)
+})
+test_that("findTransitiveCalls loop", {
+	bpgr = breakpointRanges(.testrecord(c(
+		"chr1	10000	bp1_1	N	N[chr2:100[	.	.	SVTYPE=BND;MATEID=bp1_2",
+		"chr2	100	bp1_2	N	]chr1:10000]N	.	.	SVTYPE=BND;MATEID=bp1_1",
+		"chr2	300	bp2_1	N	N[chr3:50000[	.	.	SVTYPE=BND;MATEID=bp2_2",
+		"chr3	50000	bp2_2	N	]chr2:300]N	.	.	SVTYPE=BND;MATEID=bp2_1",
+		"chr2	140	loop_1	N	]chr2:160]TTTTTN	.	.	SVTYPE=BND;MATEID=loop_2",
+		"chr2	160	loop_2	N	NTTTTT[chr2:140[	.	.	SVTYPE=BND;MATEID=loop_1",
+		"chr1	10050	imprecise_transitive_1	N	N[chr3:50030[	.	.	SVTYPE=BND;MATEID=imprecise_transitive_2;IMPRECISE;CIPOS=-60,60",
+		"chr3	50030	imprecise_transitive_2	N	]chr1:10050]N	.	.	SVTYPE=BND;MATEID=imprecise_transitive_1;IMPRECISE;CIPOS=-100,100"
+	)))
+	.us = function(df) as.data.frame(df) %>% mutate(
+		traversed_breakpoint_names=unstrsplit(traversed_breakpoint_names, ","),
+		distance_to_traversed_breakpoint=unstrsplit(CharacterList(distance_to_traversed_breakpoint), ","))
+	transdf = findTransitiveCalls(bpgr, bpgr, maximumTraversedBreakpoints=5) %>%
+		.us() %>%
+		dplyr::arrange(transitive_breakpoint_name, total_distance)
+	resultdf = DataFrame(
+		transitive_breakpoint_name=rep(c("imprecise_transitive_1", "imprecise_transitive_2"), each=4),
+		total_distance=rep(201 + 26*0:3, 2),
+		traversed_breakpoint_names=CharacterList(
+			c("bp1_1", "bp2_1"),
+			c("bp1_1", "loop_2", "bp2_1"),
+			c("bp1_1", "loop_2","loop_2", "bp2_1"),
+			c("bp1_1", "loop_2","loop_2","loop_2", "bp2_1"),
+			c("bp2_2", "bp1_2"),
+			c("bp2_2", "loop_1", "bp1_2"),
+			c("bp2_2", "loop_1", "loop_1", "bp1_2"),
+			c("bp2_2", "loop_1", "loop_1", "loop_1", "bp1_2")),
+		distance_to_traversed_breakpoint=IntegerList(
+			cumsum(c(0, 201)),
+			cumsum(c(0, 61+5, 161)),
+			cumsum(c(0, 61+5, 21+5, 161)),
+			cumsum(c(0, 61+5, 21+5, 21+5, 161)),
+			cumsum(c(0, 201)),
+			cumsum(c(0, 161+5, 61)),
+			cumsum(c(0, 161+5, 21+5, 61)),
+			cumsum(c(0, 161+5, 21+5, 21+5, 61)))) %>%
+		.us()
+	expect_equal(transdf, resultdf)
+})
+test_that(".traversable_segments", {
+	bpgr = breakpointRanges(.testrecord(c(
+		"chr1	10000	bp1_1	N	N[chr2:100[	.	.	SVTYPE=BND;MATEID=bp1_2",
+		"chr2	100	bp1_2	N	]chr1:10000]N	.	.	SVTYPE=BND;MATEID=bp1_1",
+		"chr2	300	bp2_1	N	N[chr3:50000[	.	.	SVTYPE=BND;MATEID=bp2_2",
+		"chr3	50000	bp2_2	N	]chr2:300]N	.	.	SVTYPE=BND;MATEID=bp2_1",
+		"chr2	140	loop_1	N	]chr2:160]TTTTTN	.	.	SVTYPE=BND;MATEID=loop_2",
+		"chr2	160	loop_2	N	NTTTTT[chr2:140[	.	.	SVTYPE=BND;MATEID=loop_1"
+	)))
+	#  -1 bp1    -loop-
+	#   |        |    |        +
+	#   2--------5----6--------3
+	#   -        -    +        |
+	#                          4-- bp2
+	#  100      140  160      300
+	# segments are:
+	#   2----------------------3
+	#   2-------------6
+	#            5-------------3
+	#
+	bpgr$ordinal = seq_len(length(bpgr))
+	bpgr$partnerOrdinal = partner(bpgr)$ordinal
+	segdf = .traversable_segments(bpgr, 1000)
+	expecteddf = data.frame(
+		segmentStartExternalOrdinal=c(1, 1, 6, 6),
+		segmentStartInternalOrdinal=c(2, 2, 5, 5),
+		segmentStartAdditionalLength=c(0,0,5,5),
+		segmentLength=c(61, 201, 161, 21),
+		segmentEndAdditionalLength=c(5, 0, 0, 5),
+		segmentEndInternalOrdinal=c(6, 3, 3, 6),
+		segmentEndExternalOrdinal=c(5, 4, 4, 5))
+	expecteddf = dplyr::bind_rows(expecteddf, expecteddf %>%
+		dplyr::select(
+			segmentStartExternalOrdinal=segmentEndExternalOrdinal,
+			segmentEndExternalOrdinal=segmentStartExternalOrdinal,
+			segmentLength=segmentLength,
+			segmentStartInternalOrdinal=segmentEndInternalOrdinal,
+			segmentEndInternalOrdinal=segmentStartInternalOrdinal,
+			segmentStartAdditionalLength=segmentEndAdditionalLength,
+			segmentEndAdditionalLength=segmentStartAdditionalLength))
+	expect_equal(
+		segdf %>% dplyr::arrange(segmentStartExternalOrdinal, segmentEndExternalOrdinal, segmentStartInternalOrdinal),
+		expecteddf %>% dplyr::arrange(segmentStartExternalOrdinal, segmentEndExternalOrdinal, segmentStartInternalOrdinal))
+})
+test_that("findTransitiveCalls precise", {
+	hundred_N="NNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNNN"
+	two_hundred_N=paste0(hundred_N, hundred_N)
+	# chr1 --]100]
+	#          |
+	# chr2   [200[--------]300]
+	#                       |
+	# chr3                [100[----]220]
+	#                                |
+	# chr4                         [1000[-----
+	bpgr = breakpointRanges(.testrecord(c(
+		"chr1	100	bp1_1	N	N[chr2:200[	.	.	SVTYPE=BND;MATEID=bp1_2",
+		"chr2	200	bp1_2	N	]chr1:100]N	.	.	SVTYPE=BND;MATEID=bp1_1",
+		"chr2	300	bp2_1	N	N[chr3:100[	.	.	SVTYPE=BND;MATEID=bp2_2",
+		"chr3	100	bp2_2	N	]chr2:300]N	.	.	SVTYPE=BND;MATEID=bp2_1",
+		"chr3	220	bp3_1	N	N[chr4:1000[	.	.	SVTYPE=BND;MATEID=bp3_2",
+		"chr4	1000	bp3_2	N	]chr3:220]N	.	.	SVTYPE=BND;MATEID=bp3_1",
+		paste0("chr1	100	precise_transitive_1	N	N",two_hundred_N,"[chr4:1000[	.	.	SVTYPE=BND;MATEID=precise_transitive_2"),
+		paste0("chr4	1000	precise_transitive_2	N	]chr1:100]",two_hundred_N,"N	.	.	SVTYPE=BND;MATEID=precise_transitive_1")
+	)))
+	transdf = findTransitiveCalls(bpgr, bpgr)
+	resultdf = DataFrame(
+		transitive_breakpoint_name=c("precise_transitive_1", "precise_transitive_2"),
+		total_distance=101+121,
+		traversed_breakpoint_names=CharacterList(c("bp1_1", "bp2_1", "bp3_1"), c("bp3_2", "bp2_2", "bp1_2")),
+		distance_to_traversed_breakpoint=IntegerList(c(0, 101, 101+121), c(0, 121, 121+101)))
+	expect_equal(transdf, resultdf)
+})
+test_that("findTransitiveCalls long read colo829", {
+	t_vcf = list(
+		truth = readVcf(.testfile("truthset_somaticSVs_COLO829.vcf")),
+		nanosv = readVcf(.testfile("colo829_nanoSV_truth_overlap.vcf")))
+		#gridss = readVcf(.testfile("colo829_gridss_truth_overlap.vcf")))
+	t_bpgr = list()
+	for (n in names(t_vcf)) {
+		t_bpgr[[n]] = breakpointRanges(t_vcf[[n]], inferMissingBreakends=TRUE, ignoreUnknownSymbolicAlleles=TRUE)
+	}
+	t_bpgr$nanosv$insLen = pmax(0, info(t_vcf$nanosv[t_bpgr$nanosv$sourceId])$GAP)
+	transdf = findTransitiveCalls(t_bpgr$nanosv, t_bpgr$truth)
+	expect_equal(4, nrow(transdf))
+	#transitive_files = list(
+	#	truth = "C:/dev/colo829/truthset_somaticSVs_COLO829.vcf",
+	#	pbsv = "C:/dev/colo829/COLO829.somatic.pacbio.pbsv.vcf",
+	#	nanosv = "C:/dev/colo829/COLO829.nanopore.nanosv.vcf",
+	#	sniffles = "C:/dev/colo829/COLO829.nanopore.sniffles.vcf")
+	#gridss = "C:/dev/colo829/COLO829v001-colov1_gridss_COLO829v001T.gridss.unfiltered.vcf.gz")
+})
